/*
 * @file main.c
 * @brief Est projeto implementa o exemplo 1 da secao 8.4.1/pg. 84 de KLQRUB 
 * @date 22/01/2023
 * @author Wu Shin-Ting
 */

#include "derivative.h"
#include "stdint.h"
#include "UART.h"
#include "SIM.h"
#include "MCG.h"

static uint8_t c;

void EnterVLPS(void)
{
	SMC_PMCTRL |= SMC_PMCTRL_STOPM(2);
	SCB_SCR |= SCB_SCR_SLEEPDEEP_MASK;
	asm("WFI");
}

void UART0_PutStr(uint8_t * str)
{
	uint16_t i=0;
	while(str[i] != 0)
	{
		while( !(UART0_S1&UART_S1_TDRE_MASK));
		UART0_D = str[i];
		i++;
	}
}

void UART0_IRQHandler (void)
{
	if (UART0_S1&UART_S1_RDRF_MASK)
	{
		c = UART0_D;
//		if ((UART0_S1&UART_S1_TDRE_MASK) && (UART0_S1&UART_S1_TC_MASK))
//		{
//			UART0_D = c;
//		}
	}
}

int main(void)
{
	/*
	 * Para operar no modo VLPS somente o sinal MCGIRCLK de 4MHz (Tabela 7-2 do Manual). 
	 */
	MCG_initIR4MHFLL20MH();
	
	SIM_setaFLLPLL (0);              //seta FLL
	
	/*
	 * Configurar fonte de UART0 clock e pinos
	 */
	UART0_config_basica(0b11);
	
	//enable_irq(12); set_irq_priority(12, 1);
	UART0_habilitaNVICIRQ12(0b1);

	/*
	 * Configura o modulo UART0: clock MCGIRCLK
	 */
	UART0_config_especifica (4000000, 9600, 0x0F);
	
	/*
	 * Habilitar Interrupcao que eh uma fonte de despertar
	 */
	UART0_habilitaInterruptRxTerminal();
	
	//Habilitar a entrada no modo de baixo consumo energetico
	SMC_PMPROT |= SMC_PMPROT_AVLP_MASK;

	for(;;) {	
		UART0_PutStr((uint8_t *)"\n\rPress <<e>> to enter VLPS mode with UART0 address match wake up\n\r");
		c = 0x00;
		while (!c);
		if ((c == 'e') || (c == 'E'))
		{
			UART0_PutStr((uint8_t *)"\n\rVLPS entered ...\n\rSend 0x81 or 0xAA to wake up\n\r");
			while( !(UART0_S1&UART_S1_TC_MASK));

			//Habilitar o modo de address-mark 
			UART0_C2 &= ~(UART0_C2_TE_MASK|UART0_C2_RE_MASK);
			UART0_C4 |= UART0_C4_MAEN1_MASK|UART0_C4_MAEN2_MASK;
			UART0_C1 |= UART0_C1_WAKE_MASK;
			UART0_C2 |= (UART0_C2_TE_MASK|UART0_C2_RE_MASK);

			EnterVLPS();
			UART0_C4 &= ~(UART0_C4_MAEN1_MASK|UART0_C4_MAEN2_MASK);
			UART0_C1 &= ~UART0_C1_WAKE_MASK;
		}
		else
		{
			UART0_PutStr((uint8_t *)"\n\rWrong setting. Try again.\n\r");
		}
	}
	
	return 0;
}
